# *** imports

# ** infra
from abc import ABC, abstractmethod


# *** classes


# ** class: model_contract
class ModelContract(ABC):
    '''
    The model contract interface as an abstract base class.
    '''

    pass


# ** class: repository
class Repository(ABC):
    '''
    The repository interface as an abstract base class.
    '''

    pass


# ** class: service
class Service(ABC):
    '''
    The service interface as an abstract base class.
    '''

    pass